@@ -1,10 +1,10 @@
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::ops::Deref;
 
 use quote::ToTokens;
 use syn::{Path, ReturnType, Type};
 
-use crate::bridged_type::{fn_arg_name, BridgedType, StdLibType, StructSwiftRepr, TypePosition};
+use crate::bridged_type::{fn_arg_name, BridgedType, StdLibType, TypePosition};
 use crate::codegen::generate_swift::vec::generate_vectorizable_extension;
 use crate::codegen::CodegenConfig;
 use crate::parse::{
@@ -14,9 +14,10 @@ use crate::parse::{
 use crate::parsed_extern_fn::ParsedExternFn;
 use crate::{SwiftBridgeModule, SWIFT_BRIDGE_PREFIX};
 
-mod option;
 mod vec;
 
+mod shared_struct;
+
 impl SwiftBridgeModule {
     /// Generate the corresponding Swift code for the bridging module.
     pub(crate) fn generate_swift(&self, config: &CodegenConfig) -> String {
@@ -66,16 +67,9 @@ impl SwiftBridgeModule {
         for ty in self.types.types() {
             match ty {
                 TypeDeclaration::Shared(SharedTypeDeclaration::Struct(shared_struct)) => {
-                    match shared_struct.swift_repr {
-                        StructSwiftRepr::Class => {
-                            todo!()
-                        }
-                        StructSwiftRepr::Structure => {
-                            // No need to generate any code. Swift will automatically generate a
-                            //  struct from our C header typedef that we generate for this struct.
-
-                            continue;
-                        }
+                    if let Some(swift_struct) = self.generate_shared_struct_string(shared_struct) {
+                        swift += &swift_struct;
+                        swift += "\n";
                     }
                 }
                 TypeDeclaration::Opaque(ty) => match ty.host_lang {
@@ -295,6 +289,21 @@ func {fn_name} (ptr: UnsafeMutableRawPointer) {{
     )
 }
 
+#[derive(Hash, Eq, PartialEq, Ord, PartialOrd)]
+enum SwiftFuncGenerics {
+    String,
+    Str,
+}
+
+impl SwiftFuncGenerics {
+    fn as_bound(&self) -> &'static str {
+        match self {
+            SwiftFuncGenerics::String => "GenericIntoRustString: IntoRustString",
+            SwiftFuncGenerics::Str => "GenericToRustStr: ToRustStr",
+        }
+    }
+}
+
 fn gen_func_swift_calls_rust(
     function: &ParsedExternFn,
     types: &TypeDeclarations,
@@ -349,8 +358,7 @@ fn gen_func_swift_calls_rust(
         call_rust
     } else if let Some(built_in) = function.return_ty_built_in(types) {
         built_in.convert_ffi_value_to_swift_value(
-            function.host_lang,
-            TypePosition::FnReturn,
+            TypePosition::FnReturn(function.host_lang),
             &call_rust,
         )
     } else {
@@ -402,20 +410,51 @@ fn gen_func_swift_calls_rust(
     } else {
         "return "
     };
-    for arg in function.func.sig.inputs.iter() {
-        if let Some(BridgedType::StdLib(StdLibType::Str)) = BridgedType::new_with_fn_arg(arg, types)
-        {
-            let arg_name = fn_arg_name(arg).unwrap().to_string();
 
-            call_rust = format!(
-                r#"{maybe_return}{arg}.utf8CString.withUnsafeBufferPointer({{{arg}Ptr in
+    let mut maybe_generics = HashSet::new();
+
+    for arg in function.func.sig.inputs.iter() {
+        let bridged_arg = BridgedType::new_with_fn_arg(arg, types);
+        if bridged_arg.is_none() {
+            continue;
+        }
+        let bridged_arg = bridged_arg.unwrap();
+
+        let arg_name = fn_arg_name(arg).unwrap().to_string();
+
+        if bridged_arg.contains_owned_string_recursive() {
+            maybe_generics.insert(SwiftFuncGenerics::String);
+        } else if bridged_arg.contains_ref_string_recursive() {
+            maybe_generics.insert(SwiftFuncGenerics::Str);
+        }
+
+        // TODO: Refactor to make less duplicative
+        match bridged_arg {
+            BridgedType::StdLib(StdLibType::Str) => {
+                call_rust = format!(
+                    r#"{maybe_return}{arg}.toRustStr({{ {arg}AsRustStr in
 {indentation}        {call_rust}
 {indentation}    }})"#,
-                maybe_return = maybe_return,
-                indentation = indentation,
-                arg = arg_name,
-                call_rust = call_rust
-            );
+                    maybe_return = maybe_return,
+                    indentation = indentation,
+                    arg = arg_name,
+                    call_rust = call_rust
+                );
+            }
+            BridgedType::StdLib(StdLibType::Option(briged_opt))
+                if briged_opt.ty.deref() == &BridgedType::StdLib(StdLibType::Str) =>
+            {
+                call_rust = format!(
+                    r#"{maybe_return}optionalRustStrToRustStr({arg}, {{ {arg}AsRustStr in
+{indentation}        {call_rust}
+{indentation}    }})"#,
+                    maybe_return = maybe_return,
+                    indentation = indentation,
+                    arg = arg_name,
+                    call_rust = call_rust
+                );
+            }
+            _ => {}
         }
     }
 
@@ -429,13 +468,28 @@ fn gen_func_swift_calls_rust(
         function.to_swift_return_type(types)
     };
 
+    let maybe_generics = if maybe_generics.is_empty() {
+        "".to_string()
+    } else {
+        let mut m = vec![];
+
+        let generics: Vec<SwiftFuncGenerics> = maybe_generics.into_iter().collect();
+
+        for generic in generics {
+            m.push(generic.as_bound())
+        }
+
+        format!("<{}>", m.join(", "))
+    };
+
     let func_definition = format!(
-        r#"{indentation}{maybe_static_class_func}{swift_class_func_name}({params}){maybe_ret} {{
+        r#"{indentation}{maybe_static_class_func}{swift_class_func_name}{maybe_generics}({params}){maybe_ret} {{
 {indentation}    {call_rust}
 {indentation}}}"#,
         indentation = indentation,
         maybe_static_class_func = maybe_static_class_func,
         swift_class_func_name = swift_class_func_name,
+        maybe_generics = maybe_generics,
         params = params,
         maybe_ret = maybe_return,
         call_rust = call_rust
@@ -466,8 +520,7 @@ fn gen_function_exposes_swift_to_rust(
     if let Some(built_in) = BridgedType::new_with_return_type(&func.sig.output, types) {
         call_fn = built_in.convert_swift_expression_to_ffi_compatible(
             &call_fn,
-            func.host_lang,
-            swift_bridge_path,
+            TypePosition::FnReturn(func.host_lang),
         );
 
         if let Some(associated_type) = func.associated_type.as_ref() {
@@ -516,8 +569,7 @@ func {prefixed_fn_name} ({params}){ret} {{
 
 #[cfg(test)]
 mod tests {
-    //! TODO: We're progressively moving most of these tests to `codegen_tests.rs`,
-    //!  along with their corresponding Rust token and C header generation tests.
+    //! More tests can be found in src/codegen/codegen_tests.rs and its submodules.
 
     use crate::codegen::generate_swift::CodegenConfig;
     use quote::quote;
@@ -1067,80 +1119,6 @@ func __swift_bridge__void_pointer (_ arg: UnsafeRawPointer) {
         assert_trimmed_generated_contains_trimmed_expected(&generated, &expected);
     }
 
-    /// Verify that we can return a shared struct type.
-    #[test]
-    fn extern_rust_return_shared_struct() {
-        let tokens = quote! {
-            mod ffi {
-                struct Foo;
-
-                extern "Rust" {
-                    fn get_foo () -> Foo;
-                }
-            }
-        };
-        let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
-
-        let expected = r#"
-func get_foo() -> Foo {
-    __swift_bridge__$get_foo()
-}
-"#;
-
-        assert_trimmed_generated_contains_trimmed_expected(&generated, &expected);
-    }
-
-    /// Verify that we can take a shared struct as an argument.
-    #[test]
-    fn extern_rust_shared_struct_arg() {
-        let tokens = quote! {
-            mod ffi {
-                struct Foo;
-
-                extern "Rust" {
-                    fn some_function (arg: Foo);
-                }
-            }
-        };
-        let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
-
-        let expected = r#"
-func some_function(_ arg: Foo) {
-    __swift_bridge__$some_function(arg)
-}
-"#;
-
-        assert_trimmed_generated_contains_trimmed_expected(&generated, &expected);
-    }
-
-    /// Verify that we rename shared struct arguments and return values if there is a swift_name
-    /// attribute.
-    #[test]
-    fn extern_rust_fn_uses_swift_name_for_shared_struct_attrib() {
-        let tokens = quote! {
-            mod ffi {
-                #[swift_bridge(swift_name = "Renamed")]
-                struct Foo;
-
-                extern "Rust" {
-                    fn some_function (arg: Foo) -> Foo;
-                }
-            }
-        };
-        let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
-
-        let expected = r#"
-func some_function(_ arg: Renamed) -> Renamed {
-    __swift_bridge__$some_function(arg)
-}
-"#;
-
-        assert_trimmed_generated_contains_trimmed_expected(&generated, &expected);
-    }
-
     /// Verify that we generate the correct function for an extern "Rust" fn that takes an owned
     /// opaque Swift type.
     #[test]
