@@ -7,16 +7,21 @@
 
 #![deny(missing_docs)]
 
-use crate::parse::TypeDeclarations;
 use proc_macro2::Ident;
-use syn::parse::{Parse, ParseStream};
-use syn::{Path, Token};
+use syn::Path;
 
+use crate::bridge_module_attributes::CfgAttr;
+use crate::parse::TypeDeclarations;
 use crate::parsed_extern_fn::ParsedExternFn;
 
+pub use self::bridge_macro_attributes::{SwiftBridgeModuleAttr, SwiftBridgeModuleAttrs};
+pub use self::codegen::CodegenConfig;
+
 mod errors;
 mod parse;
 
+mod bridge_macro_attributes;
+mod bridge_module_attributes;
 mod bridged_type;
 mod parsed_extern_fn;
 
@@ -56,6 +61,7 @@ pub struct SwiftBridgeModule {
     types: TypeDeclarations,
     functions: Vec<ParsedExternFn>,
     swift_bridge_path: Path,
+    cfg_attrs: Vec<CfgAttr>,
 }
 
 impl SwiftBridgeModule {
@@ -66,49 +72,6 @@ impl SwiftBridgeModule {
     }
 }
 
-/// `#\[swift_bridge::bridge(swift_bridge_path = swift_bridge\]`
-pub struct SwiftBridgeModuleAttrs {
-    #[allow(missing_docs)]
-    pub attributes: Vec<SwiftBridgeModuleAttr>,
-}
-
-/// `#\[swift_bridge::bridge(swift_bridge_path = swift_bridge\]`
-pub enum SwiftBridgeModuleAttr {
-    /// Sets the path for the actual swift bridge crate that contains different helpers such
-    /// as `RustString`.
-    SwiftBridgePath(Path),
-}
-
-impl Parse for SwiftBridgeModuleAttrs {
-    fn parse(input: ParseStream) -> syn::Result<Self> {
-        if input.is_empty() {
-            return Ok(SwiftBridgeModuleAttrs { attributes: vec![] });
-        }
-
-        let opts = syn::punctuated::Punctuated::<_, Token![,]>::parse_terminated(input)?;
-
-        Ok(SwiftBridgeModuleAttrs {
-            attributes: opts.into_iter().collect(),
-        })
-    }
-}
-
-impl Parse for SwiftBridgeModuleAttr {
-    fn parse(input: ParseStream) -> syn::Result<Self> {
-        let key: Ident = input.parse()?;
-        let _equals = input.parse::<Token![=]>()?;
-
-        let attr = match key.to_string().as_str() {
-            "swift_bridge_path" => SwiftBridgeModuleAttr::SwiftBridgePath(input.parse()?),
-            _ => {
-                return Err(syn::Error::new(input.span(), "Unknown attribute."));
-            }
-        };
-
-        Ok(attr)
-    }
-}
-
 #[cfg(test)]
 mod tests {
 
