@@ -1,6 +1,6 @@
 use crate::bridged_type::BridgedType;
 use crate::errors::{ParseError, ParseErrors};
-use crate::parse::parse_extern_mod::function_attributes::{FunctionAttr, FunctionAttributes};
+use crate::parse::parse_extern_mod::function_attributes::FunctionAttributes;
 use crate::parse::parse_extern_mod::opaque_type_attributes::{
     OpaqueTypeAttr, OpaqueTypeAttributes,
 };
@@ -13,7 +13,7 @@ use quote::ToTokens;
 use std::cmp::Ordering;
 use std::collections::HashMap;
 use std::ops::Deref;
-use syn::{FnArg, ForeignItem, ForeignItemFn, ItemForeignMod, Pat, ReturnType, Type};
+use syn::{FnArg, ForeignItem, ForeignItemFn, ItemForeignMod, Meta, Pat, ReturnType, Type};
 
 mod function_attributes;
 mod opaque_type_attributes;
@@ -71,16 +71,41 @@ impl<'a> ForeignModParser<'a> {
                     }
 
                     let mut attributes = OpaqueTypeAttributes::default();
+                    let mut doc_comment = None;
 
                     for attr in foreign_ty.attrs.iter() {
-                        let attr: OpaqueTypeAttr = attr.parse_args()?;
-                        attributes.store_attrib(attr);
+                        let attribute_name = attr.path.to_token_stream().to_string();
+
+                        match attribute_name.as_str() {
+                            "doc" => {
+                                let meta = attr.parse_meta()?;
+                                let doc = match meta {
+                                    Meta::NameValue(name_val) => match name_val.lit {
+                                        syn::Lit::Str(comment) => comment.value(),
+                                        _ => {
+                                            todo!("Push parse error that doc attribute is in incorrect format")
+                                        }
+                                    },
+                                    _ => {
+                                        todo!("Push parse error that doc attribute is in incorrect format")
+                                    }
+                                };
+
+                                doc_comment = Some(doc);
+                            }
+                            "swift_bridge" => {
+                                let attr: OpaqueTypeAttr = attr.parse_args()?;
+                                attributes.store_attrib(attr);
+                            }
+                            _ => todo!("Push unsupported attribute error."),
+                        };
                     }
 
                     let foreign_type = OpaqueForeignTypeDeclaration {
                         ty: foreign_ty.clone(),
                         host_lang,
                         already_declared: attributes.already_declared,
+                        doc_comment,
                     };
                     self.type_declarations.insert(
                         ty_name.clone(),
@@ -92,8 +117,7 @@ impl<'a> ForeignModParser<'a> {
                     let mut attributes = FunctionAttributes::default();
 
                     for attr in func.attrs.iter() {
-                        let attr: FunctionAttr = attr.parse_args()?;
-                        attributes.store_attrib(attr);
+                        attributes = attr.parse_args()?;
                     }
 
                     for arg in func.sig.inputs.iter() {
@@ -131,6 +155,7 @@ impl<'a> ForeignModParser<'a> {
                         rust_name_override: attributes.rust_name,
                         swift_name_override: attributes.swift_name,
                         into_return_type: attributes.into_return_type,
+                        args_into: attributes.args_into,
                     });
                 }
                 _ => {}
@@ -346,132 +371,6 @@ mod tests {
         }
     }
 
-    /// Verify that we can parse the into_return_type attribute from extern "Rust" blocks.
-    #[test]
-    fn parse_extern_rust_into_return_type_attribute() {
-        let tokens = quote! {
-            mod foo {
-                extern "Rust" {
-                    type Foo;
-
-                    #[swift_bridge(into_return_type)]
-                    fn some_function () -> Foo;
-                }
-            }
-        };
-
-        let module = parse_ok(tokens);
-
-        assert!(module.functions[0].into_return_type);
-    }
-
-    /// Verify that we can parse an associated function.
-    #[test]
-    fn parse_associated_function() {
-        let tokens = quote! {
-            mod foo {
-                extern "Rust" {
-                    type Foo;
-
-                    #[swift_bridge(associated_to = Foo)]
-                    fn bar ();
-                }
-            }
-        };
-
-        let module = parse_ok(tokens);
-
-        let ty = &module.types.types()[0].unwrap_opaque();
-        assert_eq!(ty.ident.to_string(), "Foo");
-
-        assert_eq!(module.functions.len(), 1,);
-    }
-
-    /// Verify that we can parse an associated function that has arguments.
-    #[test]
-    fn associated_function_with_args() {
-        let tokens = quote! {
-            mod foo {
-                extern "Rust" {
-                    type Foo;
-
-                    #[swift_bridge(associated_to = Foo)]
-                    fn bar (arg: u8);
-                }
-            }
-        };
-
-        let module = parse_ok(tokens);
-
-        let ty = &module.types.types()[0].unwrap_opaque();
-        assert_eq!(ty.ident.to_string(), "Foo");
-
-        assert_eq!(module.functions.len(), 1,);
-    }
-
-    /// Verify that we can parse an init function.
-    #[test]
-    fn initializer() {
-        let tokens = quote! {
-            mod foo {
-                extern "Rust" {
-                    type Foo;
-
-                    #[swift_bridge(init)]
-                    fn bar () -> Foo;
-                }
-            }
-        };
-
-        let module = parse_ok(tokens);
-
-        let func = &module.functions[0];
-        assert!(func.is_initializer);
-    }
-
-    /// Verify that we can parse an init function that takes inputs.
-    #[test]
-    fn initializer_with_inputs() {
-        let tokens = quote! {
-            mod foo {
-                extern "Rust" {
-                    type Foo;
-
-                    #[swift_bridge(init)]
-                    fn bar (bazz: u8) -> Foo;
-                }
-            }
-        };
-
-        let module = parse_ok(tokens);
-
-        let func = &module.functions[0];
-        assert!(func.is_initializer);
-    }
-
-    /// Verify that we push an error if the initialize type is not defined.
-    #[test]
-    fn error_if_initialized_type_not_defined() {
-        let tokens = quote! {
-            mod foo {
-                extern "Rust" {
-                    #[swift_bridge(init)]
-                    fn bar () -> Foo;
-                }
-            }
-        };
-
-        let errors = parse_errors(tokens);
-        assert_eq!(errors.len(), 1);
-
-        match &errors[0] {
-            ParseError::UndeclaredType { ty } => {
-                assert_eq!(ty.to_token_stream().to_string(), "Foo")
-            }
-            _ => panic!(),
-        }
-    }
-
     /// Verify that if a a type is defined in another foreign module block we can still use it.
     #[test]
     fn type_defined_in_another_foreign_module() {
@@ -750,4 +649,31 @@ mod tests {
                 .already_declared
         );
     }
+
+    /// Verify that we can parse a doc comment from an extern "Rust" opaque type.
+    #[test]
+    fn parse_opaque_rust_type_doc_comment() {
+        let tokens = quote! {
+            mod foo {
+                extern "Rust" {
+                    /// Some comment
+                    type AnotherType;
+                }
+            }
+        };
+
+        let module = parse_ok(tokens);
+
+        assert_eq!(
+            module
+                .types
+                .get("AnotherType")
+                .unwrap()
+                .unwrap_opaque()
+                .doc_comment
+                .as_ref()
+                .unwrap(),
+            " Some comment"
+        );
+    }
 }
